from pyxbos import *
from modbus_driver import Modbus_Driver
import os,sys
import json
import requests
import yaml
import argparse
import time
from inspect import getmembers


class ParkerDriver(Driver):
    def setup(self, cfg):
        with open('parker.yaml') as f:
            # use safe_load instead load for security reasons
            driverConfig = yaml.safe_load(f)
        self.modbus_device = Modbus_Driver(driverConfig)
        self.modbus_device.initialize_modbus()
        self.service_name = cfg['service_name']

    def read(self, requestid=None):
        output = self.modbus_device.get_data()

        # This is necessary because we need to bitwise operations to unpack the
        # the flags received from the modbus register
        regulator_flag_1 = output['regulator_flag_1']
        output['energy_saving_regulator_flag'] = bool(regulator_flag_1 & 0x0100)
        output['energy_saving_real_time_regulator_flag'] = bool(regulator_flag_1 & 0x0200)
        output['service_request_regulator_flag'] = bool(regulator_flag_1 & 0x0400)

        regulator_flag_2 = output['regulator_flag_2']
        output['on_standby_regulator_flag'] = bool(regulator_flag_2 & 0x0001)
        output['new_alarm_to_read_regulator_flag'] = bool(regulator_flag_2 & 0x0080)
        output['defrost_status_regulator_flag']	= bool(regulator_flag_2 & 0x0700)

        digital_io_status = output['digital_io_status']
        output['door_switch_input_status'] = bool(digital_io_status & 0x0001)
        output['multipurpose_input_status'] = bool(digital_io_status & 0x0002)
        output['compressor_status'] = bool(digital_io_status & 0x0100)
        output['output_defrost_status'] = bool(digital_io_status & 0x0200)
        output['fans_status'] = bool(digital_io_status & 0x0400)
        output['output_k4_status'] = bool(digital_io_status & 0x0800)

        digital_output_flags = output['digital_output_flags']
        output['energy_saving_status'] = bool(digital_output_flags & 0x0100)
        output['service_request_status'] =	bool(digital_output_flags & 0x0200)
        output['resistors_activated_by_aux_key_status'] = bool(digital_output_flags & 0x001)
        output['evaporator_valve_state'] = bool(digital_output_flags & 0x002)
        output['output_defrost_state'] = bool(digital_output_flags & 0x004)
        output['output_lux_state'] =	bool(digital_output_flags & 0x008)
        output['output_aux_state'] =	bool(digital_output_flags & 0x0010)
        output['resistors_state'] = bool(digital_output_flags & 0x0020)
        output['output_alarm_state'] =	bool(digital_output_flags & 0x0040)
        output['second_compressor_state'] =	bool(digital_output_flags & 0x0080)

        alarm_status = output['alarm_status']
        #print(format(output['alarm_status'], '#010b'))
        output['probe1_failure_alarm'] = bool(alarm_status & 0x0100)
        output['probe2_failure_alarm'] = bool(alarm_status & 0x0200)
        output['probe3_failure_alarm'] = bool(alarm_status & 0x0400)
        output['minimum_temperature_alarm'] = bool(alarm_status & 0x1000)
        output['maximum_temperture_alarm'] = bool(alarm_status & 0x2000)
        output['condensor_temperature_failure_alarm'] = bool(alarm_status & 0x4000)
        output['condensor_pre_alarm'] = bool(alarm_status & 0x8000)
        output['door_alarm'] = bool(alarm_status & 0x0004)
        output['multipurpose_input_alarm'] = bool(alarm_status & 0x0008)
        output['compressor_blocked_alarm'] = bool(alarm_status & 0x0010)
        output['power_failure_alarm'] = bool(alarm_status & 0x0020)
        output['rtc_error_alarm'] = bool(alarm_status & 0x0080)
        #print(output['rtc_error_alarm'])
        #print(format(output['rtc_error_alarm'], '#010b'))
        print(output)

        msg = xbos_pb2.XBOS(
            XBOSIoTDeviceState = iot_pb2.XBOSIoTDeviceState(
                time = int(time.time()*1e9),
                parker_state = parker_pb2.ParkerState(
                    compressor_working_hours  =   types.Double(value=output.get('compressor_working_hours',None)),
                    on_standby_status  =   types.Int64(value=output.get('on_standby_status',None)),
                    light_status  =   types.Int64(value=output.get('light_status',None)),
                    aux_output_status  =   types.Int64(value=output.get('aux_output_status',None)),
                    next_defrost_counter  =   types.Double(value=output.get('next_defrost_counter',None)),
                    door_switch_input_status  =   types.Int64(value=output.get('door_switch_input_status',None)),
                    multipurpose_input_status  =   types.Int64(value=output.get('multipurpose_input_status',None)),
                    compressor_status  =   types.Int64(value=output.get('compressor_status',None)),
                    output_defrost_status  =   types.Int64(value=output.get('output_defrost_status',None)),
                    fans_status  =   types.Int64(value=output.get('fans_status',None)),
                    output_k4_status  =   types.Int64(value=output.get('output_k4_status',None)),
                    cabinet_temperature  =   types.Double(value=output.get('cabinet_temperature',None)),
                    evaporator_temperature  =   types.Double(value=output.get('evaporator_temperature',None)),
                    auxiliary_temperature  =   types.Double(value=output.get('auxiliary_temperature',None)),
                    probe1_failure_alarm  =   types.Int64(value=output.get('probe1_failure_alarm',None)),
                    probe2_failure_alarm  =   types.Int64(value=output.get('probe2_failure_alarm',None)),
                    probe3_failure_alarm  =   types.Int64(value=output.get('probe3_failure_alarm',None)),
                    minimum_temperature_alarm  =   types.Int64(value=output.get('minimum_temperature_alarm',None)),
                    maximum_temperture_alarm  =   types.Int64(value=output.get('maximum_temperture_alarm',None)),
                    condensor_temperature_failure_alarm  =   types.Int64(value=output.get('condensor_temperature_failure_alarm',None)),
                    condensor_pre_alarm  =   types.Int64(value=output.get('condensor_pre_alarm',None)),
                    door_alarm  =   types.Int64(value=output.get('door_alarm',None)),
                    multipurpose_input_alarm  =   types.Int64(value=output.get('multipurpose_input_alarm',None)),
                    compressor_blocked_alarm  =   types.Int64(value=output.get('compressor_blocked_alarm',None)),
                    power_failure_alarm  =   types.Int64(value=output.get('power_failure_alarm',None)),
                    rtc_error_alarm  =   types.Int64(value=output.get('rtc_error_alarm',None)),
                    energy_saving_regulator_flag  =   types.Int64(value=output.get('energy_saving_regulator_flag',None)),
                    energy_saving_real_time_regulator_flag  =   types.Int64(value=output.get('energy_saving_real_time_regulator_flag',None)),
                    service_request_regulator_flag  =   types.Int64(value=output.get('service_request_regulator_flag',None)),
                    on_standby_regulator_flag  =   types.Int64(value=output.get('on_standby_regulator_flag',None)),
                    new_alarm_to_read_regulator_flag  =   types.Int64(value=output.get('new_alarm_to_read_regulator_flag',None)),
                    defrost_status_regulator_flag  =   types.Int64(value=output.get('defrost_status_regulator_flag',None)),
                    active_setpoint  =   types.Int64(value=output.get('active_setpoint',None)),
                    time_until_defrost  =   types.Int64(value=output.get('time_until_defrost',None)),
                    current_defrost_counter  =   types.Int64(value=output.get('current_defrost_counter',None)),
                    compressor_delay  =   types.Int64(value=output.get('compressor_delay',None)),
                    num_alarms_in_history  =   types.Int64(value=output.get('num_alarms_in_history',None)),
                    energy_saving_status  =   types.Int64(value=output.get('energy_saving_status',None)),
                    service_request_status  =   types.Int64(value=output.get('service_request_status',None)),
                    resistors_activated_by_aux_key_status  =   types.Int64(value=output.get('resistors_activated_by_aux_key_status',None)),
                    evaporator_valve_state  =   types.Int64(value=output.get('evaporator_valve_state',None)),
                    output_defrost_state  =   types.Int64(value=output.get('output_defrost_state',None)),
                    output_lux_state  =   types.Int64(value=output.get('output_lux_state',None)),
                    output_aux_state  =   types.Int64(value=output.get('output_aux_state',None)),
                    resistors_state  =   types.Int64(value=output.get('resistors_state',None)),
                    output_alarm_state  =   types.Int64(value=output.get('output_alarm_state',None)),
                    second_compressor_state  =   types.Int64(value=output.get('second_compressor_state',None)),
                    setpoint  =   types.Double(value=output.get('setpoint',None)),
                    r1  =   types.Double(value=output.get('r1',None)),
                    r2  =   types.Double(value=output.get('r2',None)),
                    r4  =   types.Double(value=output.get('r4',None)),
                    C0  =   types.Double(value=output.get('C0',None)),
                    C1  =   types.Double(value=output.get('C1',None)),
                    d0  =   types.Double(value=output.get('d0',None)),
                    d3  =   types.Double(value=output.get('d3',None)),
                    d5  =   types.Double(value=output.get('d5',None)),
                    d7  =   types.Double(value=output.get('d7',None)),
                    d8  =   types.Int64(value=output.get('d8',None)),
                    A0  =   types.Int64(value=output.get('A0',None)),
                    A1  =   types.Double(value=output.get('A1',None)),
                    A2  =   types.Int64(value=output.get('A2',None)),
                    A3  =   types.Int64(value=output.get('A3',None)),
                    A4  =   types.Double(value=output.get('A4',None)),
                    A5  =   types.Int64(value=output.get('A5',None)),
                    A6  =   types.Double(value=output.get('A6',None)),
                    A7  =   types.Double(value=output.get('A7',None)),
                    A8  =   types.Double(value=output.get('A8',None)),
                    A9  =   types.Double(value=output.get('A9',None)),
                    F0  =   types.Int64(value=output.get('F0',None)),
                    F1  =   types.Double(value=output.get('F1',None)),
                    F2  =   types.Int64(value=output.get('F2',None)),
                    F3  =   types.Double(value=output.get('F3',None)),
                    Hd1  =   types.Double(value=output.get('Hd1',None)),
                    Hd2  =   types.Double(value=output.get('Hd2',None)),
                    Hd3  =   types.Double(value=output.get('Hd3',None)),
                    Hd4  =   types.Double(value=output.get('Hd4',None)),
                    Hd5  =   types.Double(value=output.get('Hd5',None)),
                    Hd6  =   types.Double(value=output.get('Hd6',None))
                )
            )
        )
        print(self.report(self.service_name, msg))


if __name__ == '__main__':
    with open('parker.yaml') as f:
        # use safe_load instead load for security reasons
        driverConfig = yaml.safe_load(f)

    namespace = driverConfig['wavemq']['namespace']
    service_name = driverConfig['xbos']['service_name']
    #driver_cfg = "parker.yaml"
    print(driverConfig)

    xbos_cfg = {
        'wavemq': 'localhost:4516',
        'namespace': namespace,
        'base_resource': 'parker',
        'entity': 'parker.ent',
        'id': 'pyxbos-driver-parker-2',
        #'rate': 1800, # half hour
        'rate': 20, # 15 min
        'service_name': service_name
    }
    print(getmembers(iot_pb2))
    logging.basicConfig(level="INFO", format='%(asctime)s - %(name)s - %(message)s')
    #e = DarkSkyDriver(cfg)
    e = ParkerDriver(xbos_cfg)
    e.begin()



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: grpcserver.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='grpcserver.proto',
  package='xbospb',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x10grpcserver.proto\x12\x06xbospb\x1a\x19google/protobuf/any.proto\"\x83\x02\n\x11GRPCServerMessage\x12$\n\tUnaryCall\x18\x01 \x01(\x0b\x32\x11.xbospb.UnaryCall\x12,\n\rUnaryResponse\x18\x02 \x01(\x0b\x32\x15.xbospb.UnaryResponse\x12,\n\rStreamingCall\x18\x03 \x01(\x0b\x32\x15.xbospb.StreamingCall\x12\x34\n\x11StreamingResponse\x18\x04 \x01(\x0b\x32\x19.xbospb.StreamingResponse\x12\x1a\n\x04Ping\x18\x05 \x01(\x0b\x32\x0c.xbospb.Ping\x12\x1a\n\x04Pong\x18\x06 \x01(\x0b\x32\x0c.xbospb.Pong\"\x1e\n\x04Ping\x12\x16\n\x0e\x61\x63tive_queries\x18\x01 \x03(\x03\"\x06\n\x04Pong\"T\n\tUnaryCall\x12\x0e\n\x06method\x18\x01 \x01(\t\x12\x10\n\x08query_id\x18\x02 \x01(\x03\x12%\n\x07payload\x18\x03 \x01(\x0b\x32\x14.google.protobuf.Any\"W\n\rUnaryResponse\x12\x10\n\x08query_id\x18\x01 \x01(\x03\x12\r\n\x05\x65rror\x18\x02 \x01(\t\x12%\n\x07payload\x18\x03 \x01(\x0b\x32\x14.google.protobuf.Any\"X\n\rStreamingCall\x12\x0e\n\x06method\x18\x01 \x01(\t\x12\x10\n\x08query_id\x18\x02 \x01(\x03\x12%\n\x07payload\x18\x03 \x01(\x0b\x32\x14.google.protobuf.Any\"m\n\x11StreamingResponse\x12\x10\n\x08query_id\x18\x01 \x01(\x03\x12\r\n\x05\x65rror\x18\x02 \x01(\t\x12\x10\n\x08\x66inished\x18\x03 \x01(\x08\x12%\n\x07payload\x18\x04 \x01(\x0b\x32\x14.google.protobuf.Any\"\x17\n\nTestParams\x12\t\n\x01x\x18\x01 \x01(\t\"\x19\n\x0cTestResponse\x12\t\n\x01x\x18\x01 \x01(\t2{\n\x04Test\x12\x37\n\tTestUnary\x12\x12.xbospb.TestParams\x1a\x14.xbospb.TestResponse\"\x00\x12:\n\nTestStream\x12\x12.xbospb.TestParams\x1a\x14.xbospb.TestResponse\"\x00\x30\x01\x62\x06proto3')
  ,
  dependencies=[google_dot_protobuf_dot_any__pb2.DESCRIPTOR,])




_GRPCSERVERMESSAGE = _descriptor.Descriptor(
  name='GRPCServerMessage',
  full_name='xbospb.GRPCServerMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='UnaryCall', full_name='xbospb.GRPCServerMessage.UnaryCall', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='UnaryResponse', full_name='xbospb.GRPCServerMessage.UnaryResponse', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='StreamingCall', full_name='xbospb.GRPCServerMessage.StreamingCall', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='StreamingResponse', full_name='xbospb.GRPCServerMessage.StreamingResponse', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='Ping', full_name='xbospb.GRPCServerMessage.Ping', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='Pong', full_name='xbospb.GRPCServerMessage.Pong', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=56,
  serialized_end=315,
)


_PING = _descriptor.Descriptor(
  name='Ping',
  full_name='xbospb.Ping',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='active_queries', full_name='xbospb.Ping.active_queries', index=0,
      number=1, type=3, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=317,
  serialized_end=347,
)


_PONG = _descriptor.Descriptor(
  name='Pong',
  full_name='xbospb.Pong',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=289,
  serialized_end=295,
)


_UNARYCALL = _descriptor.Descriptor(
  name='UnaryCall',
  full_name='xbospb.UnaryCall',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='method', full_name='xbospb.UnaryCall.method', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='query_id', full_name='xbospb.UnaryCall.query_id', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='payload', full_name='xbospb.UnaryCall.payload', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=357,
  serialized_end=441,
)


_UNARYRESPONSE = _descriptor.Descriptor(
  name='UnaryResponse',
  full_name='xbospb.UnaryResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='query_id', full_name='xbospb.UnaryResponse.query_id', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='error', full_name='xbospb.UnaryResponse.error', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='payload', full_name='xbospb.UnaryResponse.payload', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=443,
  serialized_end=530,
)


_STREAMINGCALL = _descriptor.Descriptor(
  name='StreamingCall',
  full_name='xbospb.StreamingCall',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='method', full_name='xbospb.StreamingCall.method', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='query_id', full_name='xbospb.StreamingCall.query_id', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='payload', full_name='xbospb.StreamingCall.payload', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=532,
  serialized_end=620,
)


_STREAMINGRESPONSE = _descriptor.Descriptor(
  name='StreamingResponse',
  full_name='xbospb.StreamingResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='query_id', full_name='xbospb.StreamingResponse.query_id', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='error', full_name='xbospb.StreamingResponse.error', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='finished', full_name='xbospb.StreamingResponse.finished', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='payload', full_name='xbospb.StreamingResponse.payload', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=622,
  serialized_end=731,
)


_TESTPARAMS = _descriptor.Descriptor(
  name='TestParams',
  full_name='xbospb.TestParams',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='x', full_name='xbospb.TestParams.x', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=733,
  serialized_end=756,
)


_TESTRESPONSE = _descriptor.Descriptor(
  name='TestResponse',
  full_name='xbospb.TestResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='x', full_name='xbospb.TestResponse.x', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=758,
  serialized_end=783,
)

_GRPCSERVERMESSAGE.fields_by_name['UnaryCall'].message_type = _UNARYCALL
_GRPCSERVERMESSAGE.fields_by_name['UnaryResponse'].message_type = _UNARYRESPONSE
_GRPCSERVERMESSAGE.fields_by_name['StreamingCall'].message_type = _STREAMINGCALL
_GRPCSERVERMESSAGE.fields_by_name['StreamingResponse'].message_type = _STREAMINGRESPONSE
_GRPCSERVERMESSAGE.fields_by_name['Ping'].message_type = _PING
_GRPCSERVERMESSAGE.fields_by_name['Pong'].message_type = _PONG
_UNARYCALL.fields_by_name['payload'].message_type = google_dot_protobuf_dot_any__pb2._ANY
_UNARYRESPONSE.fields_by_name['payload'].message_type = google_dot_protobuf_dot_any__pb2._ANY
_STREAMINGCALL.fields_by_name['payload'].message_type = google_dot_protobuf_dot_any__pb2._ANY
_STREAMINGRESPONSE.fields_by_name['payload'].message_type = google_dot_protobuf_dot_any__pb2._ANY
DESCRIPTOR.message_types_by_name['GRPCServerMessage'] = _GRPCSERVERMESSAGE
DESCRIPTOR.message_types_by_name['Ping'] = _PING
DESCRIPTOR.message_types_by_name['Pong'] = _PONG
DESCRIPTOR.message_types_by_name['UnaryCall'] = _UNARYCALL
DESCRIPTOR.message_types_by_name['UnaryResponse'] = _UNARYRESPONSE
DESCRIPTOR.message_types_by_name['StreamingCall'] = _STREAMINGCALL
DESCRIPTOR.message_types_by_name['StreamingResponse'] = _STREAMINGRESPONSE
DESCRIPTOR.message_types_by_name['TestParams'] = _TESTPARAMS
DESCRIPTOR.message_types_by_name['TestResponse'] = _TESTRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

GRPCServerMessage = _reflection.GeneratedProtocolMessageType('GRPCServerMessage', (_message.Message,), dict(
  DESCRIPTOR = _GRPCSERVERMESSAGE,
  __module__ = 'grpcserver_pb2'
  # @@protoc_insertion_point(class_scope:xbospb.GRPCServerMessage)
  ))
_sym_db.RegisterMessage(GRPCServerMessage)

Ping = _reflection.GeneratedProtocolMessageType('Ping', (_message.Message,), dict(
  DESCRIPTOR = _PING,
  __module__ = 'grpcserver_pb2'
  # @@protoc_insertion_point(class_scope:xbospb.Ping)
  ))
_sym_db.RegisterMessage(Ping)

Pong = _reflection.GeneratedProtocolMessageType('Pong', (_message.Message,), dict(
  DESCRIPTOR = _PONG,
  __module__ = 'grpcserver_pb2'
  # @@protoc_insertion_point(class_scope:xbospb.Pong)
  ))
_sym_db.RegisterMessage(Pong)

UnaryCall = _reflection.GeneratedProtocolMessageType('UnaryCall', (_message.Message,), dict(
  DESCRIPTOR = _UNARYCALL,
  __module__ = 'grpcserver_pb2'
  # @@protoc_insertion_point(class_scope:xbospb.UnaryCall)
  ))
_sym_db.RegisterMessage(UnaryCall)

UnaryResponse = _reflection.GeneratedProtocolMessageType('UnaryResponse', (_message.Message,), dict(
  DESCRIPTOR = _UNARYRESPONSE,
  __module__ = 'grpcserver_pb2'
  # @@protoc_insertion_point(class_scope:xbospb.UnaryResponse)
  ))
_sym_db.RegisterMessage(UnaryResponse)

StreamingCall = _reflection.GeneratedProtocolMessageType('StreamingCall', (_message.Message,), dict(
  DESCRIPTOR = _STREAMINGCALL,
  __module__ = 'grpcserver_pb2'
  # @@protoc_insertion_point(class_scope:xbospb.StreamingCall)
  ))
_sym_db.RegisterMessage(StreamingCall)

StreamingResponse = _reflection.GeneratedProtocolMessageType('StreamingResponse', (_message.Message,), dict(
  DESCRIPTOR = _STREAMINGRESPONSE,
  __module__ = 'grpcserver_pb2'
  # @@protoc_insertion_point(class_scope:xbospb.StreamingResponse)
  ))
_sym_db.RegisterMessage(StreamingResponse)

TestParams = _reflection.GeneratedProtocolMessageType('TestParams', (_message.Message,), dict(
  DESCRIPTOR = _TESTPARAMS,
  __module__ = 'grpcserver_pb2'
  # @@protoc_insertion_point(class_scope:xbospb.TestParams)
  ))
_sym_db.RegisterMessage(TestParams)

TestResponse = _reflection.GeneratedProtocolMessageType('TestResponse', (_message.Message,), dict(
  DESCRIPTOR = _TESTRESPONSE,
  __module__ = 'grpcserver_pb2'
  # @@protoc_insertion_point(class_scope:xbospb.TestResponse)
  ))
_sym_db.RegisterMessage(TestResponse)



_TEST = _descriptor.ServiceDescriptor(
  name='Test',
  full_name='xbospb.Test',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=785,
  serialized_end=908,
  methods=[
  _descriptor.MethodDescriptor(
    name='TestUnary',
    full_name='xbospb.Test.TestUnary',
    index=0,
    containing_service=None,
    input_type=_TESTPARAMS,
    output_type=_TESTRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='TestStream',
    full_name='xbospb.Test.TestStream',
    index=1,
    containing_service=None,
    input_type=_TESTPARAMS,
    output_type=_TESTRESPONSE,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_TEST)

DESCRIPTOR.services_by_name['Test'] = _TEST

# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from . import wavemq_pb2 as wavemq__pb2


class WAVEMQStub(object):
  # missing associated documentation comment in .proto file
  pass

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.Publish = channel.unary_unary(
        '/mqpb.WAVEMQ/Publish',
        request_serializer=wavemq__pb2.PublishParams.SerializeToString,
        response_deserializer=wavemq__pb2.PublishResponse.FromString,
        )
    self.Subscribe = channel.unary_stream(
        '/mqpb.WAVEMQ/Subscribe',
        request_serializer=wavemq__pb2.SubscribeParams.SerializeToString,
        response_deserializer=wavemq__pb2.SubscriptionMessage.FromString,
        )
    self.Query = channel.unary_stream(
        '/mqpb.WAVEMQ/Query',
        request_serializer=wavemq__pb2.QueryParams.SerializeToString,
        response_deserializer=wavemq__pb2.QueryMessage.FromString,
        )
    self.ConnectionStatus = channel.unary_unary(
        '/mqpb.WAVEMQ/ConnectionStatus',
        request_serializer=wavemq__pb2.ConnectionStatusParams.SerializeToString,
        response_deserializer=wavemq__pb2.ConnectionStatusResponse.FromString,
        )


class WAVEMQServicer(object):
  # missing associated documentation comment in .proto file
  pass

  def Publish(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def Subscribe(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def Query(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def ConnectionStatus(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_WAVEMQServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'Publish': grpc.unary_unary_rpc_method_handler(
          servicer.Publish,
          request_deserializer=wavemq__pb2.PublishParams.FromString,
          response_serializer=wavemq__pb2.PublishResponse.SerializeToString,
      ),
      'Subscribe': grpc.unary_stream_rpc_method_handler(
          servicer.Subscribe,
          request_deserializer=wavemq__pb2.SubscribeParams.FromString,
          response_serializer=wavemq__pb2.SubscriptionMessage.SerializeToString,
      ),
      'Query': grpc.unary_stream_rpc_method_handler(
          servicer.Query,
          request_deserializer=wavemq__pb2.QueryParams.FromString,
          response_serializer=wavemq__pb2.QueryMessage.SerializeToString,
      ),
      'ConnectionStatus': grpc.unary_unary_rpc_method_handler(
          servicer.ConnectionStatus,
          request_deserializer=wavemq__pb2.ConnectionStatusParams.FromString,
          response_serializer=wavemq__pb2.ConnectionStatusResponse.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'mqpb.WAVEMQ', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))


class WAVEMQPeeringStub(object):
  # missing associated documentation comment in .proto file
  pass

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.PeerPublish = channel.unary_unary(
        '/mqpb.WAVEMQPeering/PeerPublish',
        request_serializer=wavemq__pb2.PeerPublishParams.SerializeToString,
        response_deserializer=wavemq__pb2.PeerPublishResponse.FromString,
        )
    self.PeerSubscribe = channel.unary_stream(
        '/mqpb.WAVEMQPeering/PeerSubscribe',
        request_serializer=wavemq__pb2.PeerSubscribeParams.SerializeToString,
        response_deserializer=wavemq__pb2.SubscriptionMessage.FromString,
        )
    self.PeerUnsubscribe = channel.unary_unary(
        '/mqpb.WAVEMQPeering/PeerUnsubscribe',
        request_serializer=wavemq__pb2.PeerUnsubscribeParams.SerializeToString,
        response_deserializer=wavemq__pb2.PeerUnsubscribeResponse.FromString,
        )
    self.PeerQueryRequest = channel.unary_stream(
        '/mqpb.WAVEMQPeering/PeerQueryRequest',
        request_serializer=wavemq__pb2.PeerQueryParams.SerializeToString,
        response_deserializer=wavemq__pb2.QueryMessage.FromString,
        )


class WAVEMQPeeringServicer(object):
  # missing associated documentation comment in .proto file
  pass

  def PeerPublish(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def PeerSubscribe(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def PeerUnsubscribe(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def PeerQueryRequest(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_WAVEMQPeeringServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'PeerPublish': grpc.unary_unary_rpc_method_handler(
          servicer.PeerPublish,
          request_deserializer=wavemq__pb2.PeerPublishParams.FromString,
          response_serializer=wavemq__pb2.PeerPublishResponse.SerializeToString,
      ),
      'PeerSubscribe': grpc.unary_stream_rpc_method_handler(
          servicer.PeerSubscribe,
          request_deserializer=wavemq__pb2.PeerSubscribeParams.FromString,
          response_serializer=wavemq__pb2.SubscriptionMessage.SerializeToString,
      ),
      'PeerUnsubscribe': grpc.unary_unary_rpc_method_handler(
          servicer.PeerUnsubscribe,
          request_deserializer=wavemq__pb2.PeerUnsubscribeParams.FromString,
          response_serializer=wavemq__pb2.PeerUnsubscribeResponse.SerializeToString,
      ),
      'PeerQueryRequest': grpc.unary_stream_rpc_method_handler(
          servicer.PeerQueryRequest,
          request_deserializer=wavemq__pb2.PeerQueryParams.FromString,
          response_serializer=wavemq__pb2.QueryMessage.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'mqpb.WAVEMQPeering', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))



from pyxbos.driver import *
import logging
import os
#; print os.uname()[1] 
import time
import psutil

class PsutilDriver(Driver):
    def setup(self, cfg):
        psutil.cpu_percent() # need to throw away first value
        self.hostname = os.uname()[1]
        self._log.info("# CPUS {0}".format(psutil.cpu_count()))

    def read(self, requestid=None):
        msg = xbos_pb2.XBOS(
            BasicServerStatus = system_monitor_pb2.BasicServerStatus(
                time = int(time.time()*1e9),
                hostname =              self.hostname,
                cpu_load =              [types.Double(value=x) for x in psutil.cpu_percent(interval=1, percpu=True)],
                phys_mem_available =    types.Int64(value=psutil.virtual_memory().available),
                disk_usage =            types.Double(value=psutil.disk_usage(path='/').percent),
                disk_available =        types.Double(value=psutil.disk_usage(path='/').free),
            )
        )
        self.report(self.hostname, msg)

if __name__ == '__main__':
    logging.basicConfig(level="INFO", format='%(asctime)s - %(name)s - %(message)s')

    cfg = {
        'wavemq': 'localhost:4516',
        'namespace': 'GyBnl_UdduxPIcOwkrnZfqJGQiztUWKyHj9m5zHiFHS1uQ==',
        'base_resource': 'test/system',
        'entity': 'system.ent',
        'id': 'system',
        'rate': 10,
    }
    e = PsutilDriver(cfg)
    e.begin()



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: iot.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import nullabletypes_pb2 as nullabletypes__pb2
from . import parker_pb2 as parker__pb2
from . import weather_station_pb2 as weather__station__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='iot.proto',
  package='xbospb',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\tiot.proto\x12\x06xbospb\x1a\x13nullabletypes.proto\x1a\x0cparker.proto\x1a\x15weather_station.proto\"\'\n\x03URI\x12\x11\n\tnamespace\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"c\n\x06Triple\x12\x1c\n\x07subject\x18\x01 \x01(\x0b\x32\x0b.xbospb.URI\x12\x1e\n\tpredicate\x18\x02 \x01(\x0b\x32\x0b.xbospb.URI\x12\x1b\n\x06object\x18\x03 \x01(\x0b\x32\x0b.xbospb.URI\"\xde\x02\n\x12XBOSIoTDeviceState\x12\x0c\n\x04time\x18\x01 \x01(\x04\x12\x11\n\trequestid\x18\x02 \x01(\x03\x12\r\n\x05\x65rror\x18\x03 \x01(\t\x12&\n\nthermostat\x18\x04 \x01(\x0b\x32\x12.xbospb.Thermostat\x12\x1c\n\x05meter\x18\x05 \x01(\x0b\x32\r.xbospb.Meter\x12\x1c\n\x05light\x18\x06 \x01(\x0b\x32\r.xbospb.Light\x12\x1a\n\x04\x65vse\x18\x07 \x01(\x0b\x32\x0c.xbospb.EVSE\x12)\n\x0cparker_state\x18\x08 \x01(\x0b\x32\x13.xbospb.ParkerState\x12/\n\x0fweather_current\x18\t \x01(\x0b\x32\x16.xbospb.WeatherStation\x12<\n\x12weather_prediction\x18\n \x01(\x0b\x32 .xbospb.WeatherStationPrediction\"\xb9\x01\n\x16XBOSIoTDeviceActuation\x12\x0c\n\x04time\x18\x01 \x01(\x04\x12\x11\n\trequestid\x18\x02 \x01(\x03\x12&\n\nthermostat\x18\x03 \x01(\x0b\x32\x12.xbospb.Thermostat\x12\x1c\n\x05meter\x18\x04 \x01(\x0b\x32\r.xbospb.Meter\x12\x1c\n\x05light\x18\x05 \x01(\x0b\x32\r.xbospb.Light\x12\x1a\n\x04\x65vse\x18\x06 \x01(\x0b\x32\x0c.xbospb.EVSE\"?\n\x0eXBOSIoTContext\x12\x0c\n\x04time\x18\x01 \x01(\x04\x12\x1f\n\x07\x63ontext\x18\x02 \x03(\x0b\x32\x0e.xbospb.Triple\"\xae\x03\n\nThermostat\x12#\n\x0btemperature\x18\x01 \x01(\x0b\x32\x0e.xbospb.Double\x12)\n\x11relative_humidity\x18\x02 \x01(\x0b\x32\x0e.xbospb.Double\x12\x1e\n\x08override\x18\x03 \x01(\x0b\x32\x0c.xbospb.Bool\x12\x1f\n\tfan_state\x18\x04 \x01(\x0b\x32\x0c.xbospb.Bool\x12!\n\x08\x66\x61n_mode\x18\x05 \x01(\x0e\x32\x0f.xbospb.FanMode\x12\x1e\n\x04mode\x18\x06 \x01(\x0e\x32\x10.xbospb.HVACMode\x12 \n\x05state\x18\x07 \x01(\x0e\x32\x11.xbospb.HVACState\x12*\n\x13\x65nabled_heat_stages\x18\x08 \x01(\x0b\x32\r.xbospb.Int32\x12*\n\x13\x65nabled_cool_stages\x18\t \x01(\x0b\x32\r.xbospb.Int32\x12(\n\x10heating_setpoint\x18\n \x01(\x0b\x32\x0e.xbospb.Double\x12(\n\x10\x63ooling_setpoint\x18\x0b \x01(\x0b\x32\x0e.xbospb.Double\"\xa9\x01\n\x12ThermostatSchedule\x12@\n\x0bscheduleMap\x18\x01 \x03(\x0b\x32+.xbospb.ThermostatSchedule.ScheduleMapEntry\x1aQ\n\x10ScheduleMapEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12,\n\x05value\x18\x02 \x01(\x0b\x32\x1d.xbospb.ThermostatScheduleDay:\x02\x38\x01\"H\n\x15ThermostatScheduleDay\x12/\n\x06\x62locks\x18\x01 \x03(\x0b\x32\x1f.xbospb.ThermostatScheduleBlock\"\x9b\x01\n\x17ThermostatScheduleBlock\x12(\n\x10heating_setpoint\x18\x01 \x01(\x0b\x32\x0e.xbospb.Double\x12(\n\x10\x63ooling_setpoint\x18\x02 \x01(\x0b\x32\x0e.xbospb.Double\x12\x1e\n\x04mode\x18\x03 \x01(\x0e\x32\x10.xbospb.HVACMode\x12\x0c\n\x04time\x18\x04 \x01(\t\"\x8f\x01\n\x05Meter\x12\x1d\n\x05power\x18\x01 \x01(\x0b\x32\x0e.xbospb.Double\x12\x1f\n\x07voltage\x18\x02 \x01(\x0b\x32\x0e.xbospb.Double\x12&\n\x0e\x61pparent_power\x18\x03 \x01(\x0b\x32\x0e.xbospb.Double\x12\x1e\n\x06\x65nergy\x18\x04 \x01(\x0b\x32\x0e.xbospb.Double\"G\n\x05Light\x12\x1b\n\x05state\x18\x01 \x01(\x0b\x32\x0c.xbospb.Bool\x12!\n\nbrightness\x18\x02 \x01(\x0b\x32\r.xbospb.Int64\"\xb7\x01\n\x04\x45VSE\x12%\n\rcurrent_limit\x18\x01 \x01(\x0b\x32\x0e.xbospb.Double\x12\x1f\n\x07\x63urrent\x18\x02 \x01(\x0b\x32\x0e.xbospb.Double\x12\x1f\n\x07voltage\x18\x03 \x01(\x0b\x32\x0e.xbospb.Double\x12)\n\x12\x63harging_time_left\x18\x04 \x01(\x0b\x32\r.xbospb.Int32\x12\x1b\n\x05state\x18\x05 \x01(\x0b\x32\x0c.xbospb.Bool*-\n\x07\x46\x61nMode\x12\x0b\n\x07\x46\x61nAuto\x10\x00\x12\t\n\x05\x46\x61nOn\x10\x01\x12\n\n\x06\x46\x61nOff\x10\x02*Y\n\x08HVACMode\x12\x0f\n\x0bHVACModeOff\x10\x00\x12\x14\n\x10HVACModeHeatOnly\x10\x01\x12\x14\n\x10HVACModeCoolOnly\x10\x02\x12\x10\n\x0cHVACModeAuto\x10\x03*\x81\x01\n\tHVACState\x12\x10\n\x0cHVACStateOff\x10\x00\x12\x17\n\x13HVACStateHeatStage1\x10\x01\x12\x17\n\x13HVACStateCoolStage1\x10\x02\x12\x17\n\x13HVACStateHeatStage2\x10\x03\x12\x17\n\x13HVACStateCoolStage2\x10\x04\x62\x06proto3')
  ,
  dependencies=[nullabletypes__pb2.DESCRIPTOR,parker__pb2.DESCRIPTOR,weather__station__pb2.DESCRIPTOR,])

_FANMODE = _descriptor.EnumDescriptor(
  name='FanMode',
  full_name='xbospb.FanMode',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='FanAuto', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FanOn', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FanOff', index=2, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2069,
  serialized_end=2114,
)
_sym_db.RegisterEnumDescriptor(_FANMODE)

FanMode = enum_type_wrapper.EnumTypeWrapper(_FANMODE)
_HVACMODE = _descriptor.EnumDescriptor(
  name='HVACMode',
  full_name='xbospb.HVACMode',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='HVACModeOff', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='HVACModeHeatOnly', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='HVACModeCoolOnly', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='HVACModeAuto', index=3, number=3,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2116,
  serialized_end=2205,
)
_sym_db.RegisterEnumDescriptor(_HVACMODE)

HVACMode = enum_type_wrapper.EnumTypeWrapper(_HVACMODE)
_HVACSTATE = _descriptor.EnumDescriptor(
  name='HVACState',
  full_name='xbospb.HVACState',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='HVACStateOff', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='HVACStateHeatStage1', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='HVACStateCoolStage1', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='HVACStateHeatStage2', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='HVACStateCoolStage2', index=4, number=4,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2208,
  serialized_end=2337,
)
_sym_db.RegisterEnumDescriptor(_HVACSTATE)

HVACState = enum_type_wrapper.EnumTypeWrapper(_HVACSTATE)
FanAuto = 0
FanOn = 1
FanOff = 2
HVACModeOff = 0
HVACModeHeatOnly = 1
HVACModeCoolOnly = 2
HVACModeAuto = 3
HVACStateOff = 0
HVACStateHeatStage1 = 1
HVACStateCoolStage1 = 2
HVACStateHeatStage2 = 3
HVACStateCoolStage2 = 4



_URI = _descriptor.Descriptor(
  name='URI',
  full_name='xbospb.URI',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='namespace', full_name='xbospb.URI.namespace', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='xbospb.URI.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=79,
  serialized_end=118,
)


_TRIPLE = _descriptor.Descriptor(
  name='Triple',
  full_name='xbospb.Triple',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='subject', full_name='xbospb.Triple.subject', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='predicate', full_name='xbospb.Triple.predicate', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='object', full_name='xbospb.Triple.object', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=120,
  serialized_end=219,
)


_XBOSIOTDEVICESTATE = _descriptor.Descriptor(
  name='XBOSIoTDeviceState',
  full_name='xbospb.XBOSIoTDeviceState',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='time', full_name='xbospb.XBOSIoTDeviceState.time', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='requestid', full_name='xbospb.XBOSIoTDeviceState.requestid', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='error', full_name='xbospb.XBOSIoTDeviceState.error', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='thermostat', full_name='xbospb.XBOSIoTDeviceState.thermostat', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='meter', full_name='xbospb.XBOSIoTDeviceState.meter', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='light', full_name='xbospb.XBOSIoTDeviceState.light', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='evse', full_name='xbospb.XBOSIoTDeviceState.evse', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='parker_state', full_name='xbospb.XBOSIoTDeviceState.parker_state', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='weather_current', full_name='xbospb.XBOSIoTDeviceState.weather_current', index=8,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='weather_prediction', full_name='xbospb.XBOSIoTDeviceState.weather_prediction', index=9,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=222,
  serialized_end=572,
)


_XBOSIOTDEVICEACTUATION = _descriptor.Descriptor(
  name='XBOSIoTDeviceActuation',
  full_name='xbospb.XBOSIoTDeviceActuation',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='time', full_name='xbospb.XBOSIoTDeviceActuation.time', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='requestid', full_name='xbospb.XBOSIoTDeviceActuation.requestid', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='thermostat', full_name='xbospb.XBOSIoTDeviceActuation.thermostat', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='meter', full_name='xbospb.XBOSIoTDeviceActuation.meter', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='light', full_name='xbospb.XBOSIoTDeviceActuation.light', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='evse', full_name='xbospb.XBOSIoTDeviceActuation.evse', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=575,
  serialized_end=760,
)


_XBOSIOTCONTEXT = _descriptor.Descriptor(
  name='XBOSIoTContext',
  full_name='xbospb.XBOSIoTContext',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='time', full_name='xbospb.XBOSIoTContext.time', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='context', full_name='xbospb.XBOSIoTContext.context', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=762,
  serialized_end=825,
)


_THERMOSTAT = _descriptor.Descriptor(
  name='Thermostat',
  full_name='xbospb.Thermostat',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='temperature', full_name='xbospb.Thermostat.temperature', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='relative_humidity', full_name='xbospb.Thermostat.relative_humidity', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='override', full_name='xbospb.Thermostat.override', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fan_state', full_name='xbospb.Thermostat.fan_state', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='fan_mode', full_name='xbospb.Thermostat.fan_mode', index=4,
      number=5, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='mode', full_name='xbospb.Thermostat.mode', index=5,
      number=6, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='state', full_name='xbospb.Thermostat.state', index=6,
      number=7, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='enabled_heat_stages', full_name='xbospb.Thermostat.enabled_heat_stages', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='enabled_cool_stages', full_name='xbospb.Thermostat.enabled_cool_stages', index=8,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='heating_setpoint', full_name='xbospb.Thermostat.heating_setpoint', index=9,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cooling_setpoint', full_name='xbospb.Thermostat.cooling_setpoint', index=10,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=828,
  serialized_end=1258,
)


_THERMOSTATSCHEDULE_SCHEDULEMAPENTRY = _descriptor.Descriptor(
  name='ScheduleMapEntry',
  full_name='xbospb.ThermostatSchedule.ScheduleMapEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='xbospb.ThermostatSchedule.ScheduleMapEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='xbospb.ThermostatSchedule.ScheduleMapEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1349,
  serialized_end=1430,
)

_THERMOSTATSCHEDULE = _descriptor.Descriptor(
  name='ThermostatSchedule',
  full_name='xbospb.ThermostatSchedule',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='scheduleMap', full_name='xbospb.ThermostatSchedule.scheduleMap', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_THERMOSTATSCHEDULE_SCHEDULEMAPENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1261,
  serialized_end=1430,
)


_THERMOSTATSCHEDULEDAY = _descriptor.Descriptor(
  name='ThermostatScheduleDay',
  full_name='xbospb.ThermostatScheduleDay',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='blocks', full_name='xbospb.ThermostatScheduleDay.blocks', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1432,
  serialized_end=1504,
)


_THERMOSTATSCHEDULEBLOCK = _descriptor.Descriptor(
  name='ThermostatScheduleBlock',
  full_name='xbospb.ThermostatScheduleBlock',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='heating_setpoint', full_name='xbospb.ThermostatScheduleBlock.heating_setpoint', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cooling_setpoint', full_name='xbospb.ThermostatScheduleBlock.cooling_setpoint', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='mode', full_name='xbospb.ThermostatScheduleBlock.mode', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='time', full_name='xbospb.ThermostatScheduleBlock.time', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1507,
  serialized_end=1662,
)


_METER = _descriptor.Descriptor(
  name='Meter',
  full_name='xbospb.Meter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='power', full_name='xbospb.Meter.power', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='voltage', full_name='xbospb.Meter.voltage', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='apparent_power', full_name='xbospb.Meter.apparent_power', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='energy', full_name='xbospb.Meter.energy', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1665,
  serialized_end=1808,
)


_LIGHT = _descriptor.Descriptor(
  name='Light',
  full_name='xbospb.Light',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='state', full_name='xbospb.Light.state', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='brightness', full_name='xbospb.Light.brightness', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1810,
  serialized_end=1881,
)


_EVSE = _descriptor.Descriptor(
  name='EVSE',
  full_name='xbospb.EVSE',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='current_limit', full_name='xbospb.EVSE.current_limit', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='current', full_name='xbospb.EVSE.current', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='voltage', full_name='xbospb.EVSE.voltage', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='charging_time_left', full_name='xbospb.EVSE.charging_time_left', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='state', full_name='xbospb.EVSE.state', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1884,
  serialized_end=2067,
)

_TRIPLE.fields_by_name['subject'].message_type = _URI
_TRIPLE.fields_by_name['predicate'].message_type = _URI
_TRIPLE.fields_by_name['object'].message_type = _URI
_XBOSIOTDEVICESTATE.fields_by_name['thermostat'].message_type = _THERMOSTAT
_XBOSIOTDEVICESTATE.fields_by_name['meter'].message_type = _METER
_XBOSIOTDEVICESTATE.fields_by_name['light'].message_type = _LIGHT
_XBOSIOTDEVICESTATE.fields_by_name['evse'].message_type = _EVSE
_XBOSIOTDEVICESTATE.fields_by_name['parker_state'].message_type = parker__pb2._PARKERSTATE
_XBOSIOTDEVICESTATE.fields_by_name['weather_current'].message_type = weather__station__pb2._WEATHERSTATION
_XBOSIOTDEVICESTATE.fields_by_name['weather_prediction'].message_type = weather__station__pb2._WEATHERSTATIONPREDICTION
_XBOSIOTDEVICEACTUATION.fields_by_name['thermostat'].message_type = _THERMOSTAT
_XBOSIOTDEVICEACTUATION.fields_by_name['meter'].message_type = _METER
_XBOSIOTDEVICEACTUATION.fields_by_name['light'].message_type = _LIGHT
_XBOSIOTDEVICEACTUATION.fields_by_name['evse'].message_type = _EVSE
_XBOSIOTCONTEXT.fields_by_name['context'].message_type = _TRIPLE
_THERMOSTAT.fields_by_name['temperature'].message_type = nullabletypes__pb2._DOUBLE
_THERMOSTAT.fields_by_name['relative_humidity'].message_type = nullabletypes__pb2._DOUBLE
_THERMOSTAT.fields_by_name['override'].message_type = nullabletypes__pb2._BOOL
_THERMOSTAT.fields_by_name['fan_state'].message_type = nullabletypes__pb2._BOOL
_THERMOSTAT.fields_by_name['fan_mode'].enum_type = _FANMODE
_THERMOSTAT.fields_by_name['mode'].enum_type = _HVACMODE
_THERMOSTAT.fields_by_name['state'].enum_type = _HVACSTATE
_THERMOSTAT.fields_by_name['enabled_heat_stages'].message_type = nullabletypes__pb2._INT32
_THERMOSTAT.fields_by_name['enabled_cool_stages'].message_type = nullabletypes__pb2._INT32
_THERMOSTAT.fields_by_name['heating_setpoint'].message_type = nullabletypes__pb2._DOUBLE
_THERMOSTAT.fields_by_name['cooling_setpoint'].message_type = nullabletypes__pb2._DOUBLE
_THERMOSTATSCHEDULE_SCHEDULEMAPENTRY.fields_by_name['value'].message_type = _THERMOSTATSCHEDULEDAY
_THERMOSTATSCHEDULE_SCHEDULEMAPENTRY.containing_type = _THERMOSTATSCHEDULE
_THERMOSTATSCHEDULE.fields_by_name['scheduleMap'].message_type = _THERMOSTATSCHEDULE_SCHEDULEMAPENTRY
_THERMOSTATSCHEDULEDAY.fields_by_name['blocks'].message_type = _THERMOSTATSCHEDULEBLOCK
_THERMOSTATSCHEDULEBLOCK.fields_by_name['heating_setpoint'].message_type = nullabletypes__pb2._DOUBLE
_THERMOSTATSCHEDULEBLOCK.fields_by_name['cooling_setpoint'].message_type = nullabletypes__pb2._DOUBLE
_THERMOSTATSCHEDULEBLOCK.fields_by_name['mode'].enum_type = _HVACMODE
_METER.fields_by_name['power'].message_type = nullabletypes__pb2._DOUBLE
_METER.fields_by_name['voltage'].message_type = nullabletypes__pb2._DOUBLE
_METER.fields_by_name['apparent_power'].message_type = nullabletypes__pb2._DOUBLE
_METER.fields_by_name['energy'].message_type = nullabletypes__pb2._DOUBLE
_LIGHT.fields_by_name['state'].message_type = nullabletypes__pb2._BOOL
_LIGHT.fields_by_name['brightness'].message_type = nullabletypes__pb2._INT64
_EVSE.fields_by_name['current_limit'].message_type = nullabletypes__pb2._DOUBLE
_EVSE.fields_by_name['current'].message_type = nullabletypes__pb2._DOUBLE
_EVSE.fields_by_name['voltage'].message_type = nullabletypes__pb2._DOUBLE
_EVSE.fields_by_name['charging_time_left'].message_type = nullabletypes__pb2._INT32
_EVSE.fields_by_name['state'].message_type = nullabletypes__pb2._BOOL
DESCRIPTOR.message_types_by_name['URI'] = _URI
DESCRIPTOR.message_types_by_name['Triple'] = _TRIPLE
DESCRIPTOR.message_types_by_name['XBOSIoTDeviceState'] = _XBOSIOTDEVICESTATE
DESCRIPTOR.message_types_by_name['XBOSIoTDeviceActuation'] = _XBOSIOTDEVICEACTUATION
DESCRIPTOR.message_types_by_name['XBOSIoTContext'] = _XBOSIOTCONTEXT
DESCRIPTOR.message_types_by_name['Thermostat'] = _THERMOSTAT
DESCRIPTOR.message_types_by_name['ThermostatSchedule'] = _THERMOSTATSCHEDULE
DESCRIPTOR.message_types_by_name['ThermostatScheduleDay'] = _THERMOSTATSCHEDULEDAY
DESCRIPTOR.message_types_by_name['ThermostatScheduleBlock'] = _THERMOSTATSCHEDULEBLOCK
DESCRIPTOR.message_types_by_name['Meter'] = _METER
DESCRIPTOR.message_types_by_name['Light'] = _LIGHT
DESCRIPTOR.message_types_by_name['EVSE'] = _EVSE
DESCRIPTOR.enum_types_by_name['FanMode'] = _FANMODE
DESCRIPTOR.enum_types_by_name['HVACMode'] = _HVACMODE
DESCRIPTOR.enum_types_by_name['HVACState'] = _HVACSTATE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

URI = _reflection.GeneratedProtocolMessageType('URI', (_message.Message,), dict(
  DESCRIPTOR = _URI,
  __module__ = 'iot_pb2'
  # @@protoc_insertion_point(class_scope:xbospb.URI)
  ))
_sym_db.RegisterMessage(URI)

Triple = _reflection.GeneratedProtocolMessageType('Triple', (_message.Message,), dict(
  DESCRIPTOR = _TRIPLE,
  __module__ = 'iot_pb2'
  # @@protoc_insertion_point(class_scope:xbospb.Triple)
  ))
_sym_db.RegisterMessage(Triple)

XBOSIoTDeviceState = _reflection.GeneratedProtocolMessageType('XBOSIoTDeviceState', (_message.Message,), dict(
  DESCRIPTOR = _XBOSIOTDEVICESTATE,
  __module__ = 'iot_pb2'
  # @@protoc_insertion_point(class_scope:xbospb.XBOSIoTDeviceState)
  ))
_sym_db.RegisterMessage(XBOSIoTDeviceState)

XBOSIoTDeviceActuation = _reflection.GeneratedProtocolMessageType('XBOSIoTDeviceActuation', (_message.Message,), dict(
  DESCRIPTOR = _XBOSIOTDEVICEACTUATION,
  __module__ = 'iot_pb2'
  # @@protoc_insertion_point(class_scope:xbospb.XBOSIoTDeviceActuation)
  ))
_sym_db.RegisterMessage(XBOSIoTDeviceActuation)

XBOSIoTContext = _reflection.GeneratedProtocolMessageType('XBOSIoTContext', (_message.Message,), dict(
  DESCRIPTOR = _XBOSIOTCONTEXT,
  __module__ = 'iot_pb2'
  # @@protoc_insertion_point(class_scope:xbospb.XBOSIoTContext)
  ))
_sym_db.RegisterMessage(XBOSIoTContext)

Thermostat = _reflection.GeneratedProtocolMessageType('Thermostat', (_message.Message,), dict(
  DESCRIPTOR = _THERMOSTAT,
  __module__ = 'iot_pb2'
  # @@protoc_insertion_point(class_scope:xbospb.Thermostat)
  ))
_sym_db.RegisterMessage(Thermostat)

ThermostatSchedule = _reflection.GeneratedProtocolMessageType('ThermostatSchedule', (_message.Message,), dict(

  ScheduleMapEntry = _reflection.GeneratedProtocolMessageType('ScheduleMapEntry', (_message.Message,), dict(
    DESCRIPTOR = _THERMOSTATSCHEDULE_SCHEDULEMAPENTRY,
    __module__ = 'iot_pb2'
    # @@protoc_insertion_point(class_scope:xbospb.ThermostatSchedule.ScheduleMapEntry)
    ))
  ,
  DESCRIPTOR = _THERMOSTATSCHEDULE,
  __module__ = 'iot_pb2'
  # @@protoc_insertion_point(class_scope:xbospb.ThermostatSchedule)
  ))
_sym_db.RegisterMessage(ThermostatSchedule)
_sym_db.RegisterMessage(ThermostatSchedule.ScheduleMapEntry)

ThermostatScheduleDay = _reflection.GeneratedProtocolMessageType('ThermostatScheduleDay', (_message.Message,), dict(
  DESCRIPTOR = _THERMOSTATSCHEDULEDAY,
  __module__ = 'iot_pb2'
  # @@protoc_insertion_point(class_scope:xbospb.ThermostatScheduleDay)
  ))
_sym_db.RegisterMessage(ThermostatScheduleDay)

ThermostatScheduleBlock = _reflection.GeneratedProtocolMessageType('ThermostatScheduleBlock', (_message.Message,), dict(
  DESCRIPTOR = _THERMOSTATSCHEDULEBLOCK,
  __module__ = 'iot_pb2'
  # @@protoc_insertion_point(class_scope:xbospb.ThermostatScheduleBlock)
  ))
_sym_db.RegisterMessage(ThermostatScheduleBlock)

Meter = _reflection.GeneratedProtocolMessageType('Meter', (_message.Message,), dict(
  DESCRIPTOR = _METER,
  __module__ = 'iot_pb2'
  # @@protoc_insertion_point(class_scope:xbospb.Meter)
  ))
_sym_db.RegisterMessage(Meter)

Light = _reflection.GeneratedProtocolMessageType('Light', (_message.Message,), dict(
  DESCRIPTOR = _LIGHT,
  __module__ = 'iot_pb2'
  # @@protoc_insertion_point(class_scope:xbospb.Light)
  ))
_sym_db.RegisterMessage(Light)

EVSE = _reflection.GeneratedProtocolMessageType('EVSE', (_message.Message,), dict(
  DESCRIPTOR = _EVSE,
  __module__ = 'iot_pb2'
  # @@protoc_insertion_point(class_scope:xbospb.EVSE)
  ))
_sym_db.RegisterMessage(EVSE)


_THERMOSTATSCHEDULE_SCHEDULEMAPENTRY._options = None
# @@protoc_insertion_point(module_scope)



from pyxbos.driver import *
from pyxbos import weather_station_pb2
import os,sys
import json
import requests
import yaml
import argparse
import logging

class WeatherCurrentDriver(Driver):
    def setup(self, cfg):
        self.baseurl = cfg['darksky']['url']
        self.apikey = cfg['darksky']['apikey']
        self.coords = cfg['darksky']['coordinates']
        self.url = self.baseurl + self.apikey + '/' + self.coords

    def read(self, requestid=None):
        response = requests.get(self.url)
        json_data = json.loads(response.text)
        if 'currently' not in json_data: return

        logging.info("currently {0}".format(json_data['currently']))
        print(json_data['currently'])
        output = {}
        for key, value in json_data['currently'].items():
            output[key] = value

        #print(json_data['currently'])
        if 'humidity' in output:
            output['humidity'] *= 100 # change from decimal to percent

        msg = xbos_pb2.XBOS(
            XBOSIoTDeviceState = iot_pb2.XBOSIoTDeviceState(
                time = int(time.time()*1e9),
                weather_current = weather_station_pb2.WeatherStation(
                    time  =   types.Int64(value=output.get('time',None)),
                    icon  =  output.get('icon',None),
                    nearestStormDistance  =   types.Double(value=output.get('nearestStormDistance',None)),
                    nearestStormBearing  =   types.Double(value=output.get('nearestStormBearing',None)),
                    precipIntensity  =   types.Double(value=output.get('precipIntensity',None)),
                    precipIntensityError  =   types.Double(value=output.get('precipIntensityError',None)),
                    precipProbability  =   types.Double(value=output.get('precipProbability',None)),
                    precipType  =  output.get('precipType',None),
                    temperature  =   types.Double(value=output.get('temperature',None)),
                    apparentTemperature  =   types.Double(value=output.get('apparentTemperature',None)),
                    dewPoint  =   types.Double(value=output.get('dewPoint',None)),
                    humidity  =   types.Double(value=output.get('humidity',None)),
                    pressure  =   types.Double(value=output.get('pressure',None)),
                    windSpeed  =   types.Double(value=output.get('windSpeed',None)),
                    windGust  =   types.Double(value=output.get('windGust',None)),
                    windBearing  =   types.Double(value=output.get('windBearing',None)),
                    cloudCover  =   types.Double(value=output.get('cloudCover',None)),
                    uvIndex  =   types.Double(value=output.get('uvIndex',None)),
                    visibility  =   types.Double(value=output.get('visibility',None)),
                    ozone  =   types.Double(value=output.get('ozone',None)),
                )
            )
        )
        self.report(self.coords, msg)


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument("config_file", help="config file with api key as well as namespace")
    parser.add_argument("ent_file", help="entity file")
    args = parser.parse_args()
    config_file = args.config_file
    ent_file = args.ent_file

    with open(config_file) as f:
        # use safe_load instead load for security reasons
        driverConfig = yaml.safe_load(f)

    namespace = driverConfig['wavemq']['namespace']
    api = driverConfig['dark_sky']['api']
    cfg = {
        'darksky': {
            'apikey': api,
            'url': 'https://api.darksky.net/forecast/',
            'coordinates': '40.5301,-124.0000' # Should be near BLR
        },
        'wavemq': 'localhost:4516',
        'namespace': namespace,
        'base_resource': 'weather_current',
        'entity': ent_file,
        'id': 'pyxbos-driver-current-1',
        #'rate': 1800, # half hour
        'rate': 20, # 15 min
    }
    logging.basicConfig(level="INFO", format='%(asctime)s - %(name)s - %(message)s')
    current_driver = WeatherCurrentDriver(cfg)
    current_driver.begin()



import os
import logging
import time
import threading
import grpc
import pickle
import base64

from . import nullabletypes_pb2 as types
from .eapi_pb2 import *
from .wavemq_pb2 import *
from .wavemq_pb2_grpc import *
from . import xbos_pb2
from . import iot_pb2
from . import system_monitor_pb2
from pyxbos.exceptions import *

import asyncio

class Driver:
    """Base class encapsulating driver report functionality"""
    def __init__(self, cfg):
        self._log = logging.getLogger(__name__)

        self._log.info("Reading config {0}".format(str(cfg)))

        # check defaults
        if 'wavemq' not in cfg:
            cfg['wavemq'] = 'localhost:4516'
        if 'waved' not in cfg:
            cfg['waved'] = 'localhost:410'
        if 'entity' not in cfg:
            if 'WAVE_DEFAULT_ENTITY' in os.environ:
                cfg['entity'] = os.environ['WAVE_DEFAULT_ENTITY']
            else:
                raise ConfigMissingError('entity', extra="And no WAVE_DEFAULT_ENTITY in environment")
        if 'id' not in cfg:
            raise ConfigMissingError('id')
        if 'namespace' not in cfg:
            raise ConfigMissingError('namespace')
        if 'base_resource' not in cfg:
            raise ConfigMissingError('base_resource')

        self._cfg = cfg

        # connect to the wavemq agent
        self._log.info("Connecting to wavemq agent at {0}".format(cfg['wavemq']))
        self.connect()
        self._log.info("Connected to wavemq")

        # load the wave entity
        self._log.info("Loading wave entity {0}".format(cfg['entity']))
        self._ent = open(self._cfg['entity'],'rb').read()
        self._perspective = Perspective(
            entitySecret=EntitySecret(DER=self._ent),
        )
        self._namespace = b64decode(self._cfg['namespace'])
        self._uri = self._cfg['base_resource']

    def connect(self):
        # connect to wavemq agent
        wavemq_channel = grpc.insecure_channel(self._cfg['wavemq'])
        self.cl = WAVEMQStub(wavemq_channel)

    def begin(self):
        # call self.setup
        self._log.info("Run driver setup")
        self.setup(self._cfg)

        # subscribe to the write uri
        writeuri = self._cfg['base_resource']+'/write/*'
        self._log.info("Subscribe to write URI {0}".format(writeuri))

        sub = self.cl.Subscribe(SubscribeParams(
            perspective=self._perspective,
            namespace=self._namespace,
            uri=writeuri,
            identifier=self._cfg['id'],
            expiry=120,
        ))

        loop = asyncio.get_event_loop()
        
        async def _doread(requestid=None):
            self.read(requestid=requestid)

        async def readloop():
            while True:
                await _doread()
                await asyncio.sleep(self._cfg['rate'])

        # this runs in a thread
        def writeloop():
            # create an event loop because we're in a new thread
            loop = asyncio.new_event_loop()
            self._log.info("write loop")
            for msg in sub:
                if len(msg.error.message) > 0:
                    self._log.error("Get actuation message. Error {0}".format(msg.error.message))
                    continue
                m = msg.message
                now = int(time.time()*1e9)
                # seconds
                since = (now - m.timestamps[-1]) / 1.e9
                #print('timestamps', m.timestamps, 'since', since)
                #print('drops', m.drops)
                #print('resource', m.tbs.uri)
                #print('pos', len(m.tbs.payload))
                for po in m.tbs.payload:
                    print('po', po.schema, len(po.content))
                    x = xbos_pb2.XBOS.FromString(po.content)
                    try:
                        self.write(m.tbs.uri, since, x)
                    except Exception as e:
                        print('error write', e)

        # start thread
        t = threading.Thread(target=writeloop)
        t.start()


        asyncio.ensure_future(readloop())
        try:
            loop.run_forever()
        finally:
            loop.close()

    def report(self, resource, msg):
        po = PayloadObject(
            schema = "xbosproto/XBOS",
            content = msg.SerializeToString(),
        )
        self._log.info("Publishing on %s", self._uri+"/"+resource)
        try:
            x = self.cl.Publish(PublishParams(
                perspective=self._perspective,
                namespace=self._namespace,
                uri = self._uri+"/"+resource,
                content = [po],
            ))
            if not x:
                self._log.error("Error reading: {0}".format(x))
                print('x>',x)
        except Exception as e:
            self._log.error("Error reading: {0}".format(e))

def b64decode(e):
    return base64.b64decode(e, altchars=bytes('-_', 'utf8'))
def b64encode(e):
    return base64.b64encode(e, altchars=bytes('-_', 'utf8'))



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: weather_station.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import nullabletypes_pb2 as nullabletypes__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='weather_station.proto',
  package='xbospb',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x15weather_station.proto\x12\x06xbospb\x1a\x13nullabletypes.proto\"\xd6\x05\n\x0eWeatherStation\x12\x1b\n\x04time\x18\x01 \x01(\x0b\x32\r.xbospb.Int64\x12\x0c\n\x04icon\x18\x02 \x01(\t\x12,\n\x14nearestStormDistance\x18\x03 \x01(\x0b\x32\x0e.xbospb.Double\x12+\n\x13nearestStormBearing\x18\x04 \x01(\x0b\x32\x0e.xbospb.Double\x12\'\n\x0fprecipIntensity\x18\x05 \x01(\x0b\x32\x0e.xbospb.Double\x12,\n\x14precipIntensityError\x18\x06 \x01(\x0b\x32\x0e.xbospb.Double\x12)\n\x11precipProbability\x18\x07 \x01(\x0b\x32\x0e.xbospb.Double\x12\x12\n\nprecipType\x18\x08 \x01(\t\x12#\n\x0btemperature\x18\t \x01(\x0b\x32\x0e.xbospb.Double\x12+\n\x13\x61pparentTemperature\x18\n \x01(\x0b\x32\x0e.xbospb.Double\x12 \n\x08\x64\x65wPoint\x18\x0b \x01(\x0b\x32\x0e.xbospb.Double\x12 \n\x08humidity\x18\x0c \x01(\x0b\x32\x0e.xbospb.Double\x12 \n\x08pressure\x18\r \x01(\x0b\x32\x0e.xbospb.Double\x12!\n\twindSpeed\x18\x0e \x01(\x0b\x32\x0e.xbospb.Double\x12 \n\x08windGust\x18\x0f \x01(\x0b\x32\x0e.xbospb.Double\x12#\n\x0bwindBearing\x18\x10 \x01(\x0b\x32\x0e.xbospb.Double\x12\"\n\ncloudCover\x18\x11 \x01(\x0b\x32\x0e.xbospb.Double\x12\x1f\n\x07uvIndex\x18\x12 \x01(\x0b\x32\x0e.xbospb.Double\x12\"\n\nvisibility\x18\x13 \x01(\x0b\x32\x0e.xbospb.Double\x12\x1d\n\x05ozone\x18\x14 \x01(\x0b\x32\x0e.xbospb.Double\"\xaf\x01\n\x18WeatherStationPrediction\x12@\n\x0bpredictions\x18\x01 \x03(\x0b\x32+.xbospb.WeatherStationPrediction.Prediction\x1aQ\n\nPrediction\x12\x17\n\x0fprediction_time\x18\x01 \x01(\x04\x12*\n\nprediction\x18\x02 \x01(\x0b\x32\x16.xbospb.WeatherStationb\x06proto3')
  ,
  dependencies=[nullabletypes__pb2.DESCRIPTOR,])




_WEATHERSTATION = _descriptor.Descriptor(
  name='WeatherStation',
  full_name='xbospb.WeatherStation',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='time', full_name='xbospb.WeatherStation.time', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='icon', full_name='xbospb.WeatherStation.icon', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='nearestStormDistance', full_name='xbospb.WeatherStation.nearestStormDistance', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='nearestStormBearing', full_name='xbospb.WeatherStation.nearestStormBearing', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='precipIntensity', full_name='xbospb.WeatherStation.precipIntensity', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='precipIntensityError', full_name='xbospb.WeatherStation.precipIntensityError', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='precipProbability', full_name='xbospb.WeatherStation.precipProbability', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='precipType', full_name='xbospb.WeatherStation.precipType', index=7,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='temperature', full_name='xbospb.WeatherStation.temperature', index=8,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='apparentTemperature', full_name='xbospb.WeatherStation.apparentTemperature', index=9,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='dewPoint', full_name='xbospb.WeatherStation.dewPoint', index=10,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='humidity', full_name='xbospb.WeatherStation.humidity', index=11,
      number=12, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='pressure', full_name='xbospb.WeatherStation.pressure', index=12,
      number=13, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='windSpeed', full_name='xbospb.WeatherStation.windSpeed', index=13,
      number=14, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='windGust', full_name='xbospb.WeatherStation.windGust', index=14,
      number=15, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='windBearing', full_name='xbospb.WeatherStation.windBearing', index=15,
      number=16, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cloudCover', full_name='xbospb.WeatherStation.cloudCover', index=16,
      number=17, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='uvIndex', full_name='xbospb.WeatherStation.uvIndex', index=17,
      number=18, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='visibility', full_name='xbospb.WeatherStation.visibility', index=18,
      number=19, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ozone', full_name='xbospb.WeatherStation.ozone', index=19,
      number=20, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=55,
  serialized_end=781,
)


_WEATHERSTATIONPREDICTION_PREDICTION = _descriptor.Descriptor(
  name='Prediction',
  full_name='xbospb.WeatherStationPrediction.Prediction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='prediction_time', full_name='xbospb.WeatherStationPrediction.Prediction.prediction_time', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='prediction', full_name='xbospb.WeatherStationPrediction.Prediction.prediction', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=878,
  serialized_end=959,
)

_WEATHERSTATIONPREDICTION = _descriptor.Descriptor(
  name='WeatherStationPrediction',
  full_name='xbospb.WeatherStationPrediction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='predictions', full_name='xbospb.WeatherStationPrediction.predictions', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_WEATHERSTATIONPREDICTION_PREDICTION, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=784,
  serialized_end=959,
)

_WEATHERSTATION.fields_by_name['time'].message_type = nullabletypes__pb2._INT64
_WEATHERSTATION.fields_by_name['nearestStormDistance'].message_type = nullabletypes__pb2._DOUBLE
_WEATHERSTATION.fields_by_name['nearestStormBearing'].message_type = nullabletypes__pb2._DOUBLE
_WEATHERSTATION.fields_by_name['precipIntensity'].message_type = nullabletypes__pb2._DOUBLE
_WEATHERSTATION.fields_by_name['precipIntensityError'].message_type = nullabletypes__pb2._DOUBLE
_WEATHERSTATION.fields_by_name['precipProbability'].message_type = nullabletypes__pb2._DOUBLE
_WEATHERSTATION.fields_by_name['temperature'].message_type = nullabletypes__pb2._DOUBLE
_WEATHERSTATION.fields_by_name['apparentTemperature'].message_type = nullabletypes__pb2._DOUBLE
_WEATHERSTATION.fields_by_name['dewPoint'].message_type = nullabletypes__pb2._DOUBLE
_WEATHERSTATION.fields_by_name['humidity'].message_type = nullabletypes__pb2._DOUBLE
_WEATHERSTATION.fields_by_name['pressure'].message_type = nullabletypes__pb2._DOUBLE
_WEATHERSTATION.fields_by_name['windSpeed'].message_type = nullabletypes__pb2._DOUBLE
_WEATHERSTATION.fields_by_name['windGust'].message_type = nullabletypes__pb2._DOUBLE
_WEATHERSTATION.fields_by_name['windBearing'].message_type = nullabletypes__pb2._DOUBLE
_WEATHERSTATION.fields_by_name['cloudCover'].message_type = nullabletypes__pb2._DOUBLE
_WEATHERSTATION.fields_by_name['uvIndex'].message_type = nullabletypes__pb2._DOUBLE
_WEATHERSTATION.fields_by_name['visibility'].message_type = nullabletypes__pb2._DOUBLE
_WEATHERSTATION.fields_by_name['ozone'].message_type = nullabletypes__pb2._DOUBLE
_WEATHERSTATIONPREDICTION_PREDICTION.fields_by_name['prediction'].message_type = _WEATHERSTATION
_WEATHERSTATIONPREDICTION_PREDICTION.containing_type = _WEATHERSTATIONPREDICTION
_WEATHERSTATIONPREDICTION.fields_by_name['predictions'].message_type = _WEATHERSTATIONPREDICTION_PREDICTION
DESCRIPTOR.message_types_by_name['WeatherStation'] = _WEATHERSTATION
DESCRIPTOR.message_types_by_name['WeatherStationPrediction'] = _WEATHERSTATIONPREDICTION
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

WeatherStation = _reflection.GeneratedProtocolMessageType('WeatherStation', (_message.Message,), dict(
  DESCRIPTOR = _WEATHERSTATION,
  __module__ = 'weather_station_pb2'
  # @@protoc_insertion_point(class_scope:xbospb.WeatherStation)
  ))
_sym_db.RegisterMessage(WeatherStation)

WeatherStationPrediction = _reflection.GeneratedProtocolMessageType('WeatherStationPrediction', (_message.Message,), dict(

  Prediction = _reflection.GeneratedProtocolMessageType('Prediction', (_message.Message,), dict(
    DESCRIPTOR = _WEATHERSTATIONPREDICTION_PREDICTION,
    __module__ = 'weather_station_pb2'
    # @@protoc_insertion_point(class_scope:xbospb.WeatherStationPrediction.Prediction)
    ))
  ,
  DESCRIPTOR = _WEATHERSTATIONPREDICTION,
  __module__ = 'weather_station_pb2'
  # @@protoc_insertion_point(class_scope:xbospb.WeatherStationPrediction)
  ))
_sym_db.RegisterMessage(WeatherStationPrediction)
_sym_db.RegisterMessage(WeatherStationPrediction.Prediction)


# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['pyxbos', 'pyxbos.drivers']

package_data = \
{'': ['*'],
 'pyxbos': ['wave/*', 'wavemq/*'],
 'pyxbos.drivers': ['dark_sky/api',
                    'dark_sky/api',
                    'dark_sky/api',
                    'dark_sky/config.yaml',
                    'dark_sky/config.yaml',
                    'dark_sky/config.yaml',
                    'dark_sky/dark_sky.py',
                    'dark_sky/dark_sky.py',
                    'dark_sky/dark_sky.py',
                    'hue/hue.py',
                    'hue/hue.py',
                    'hue/requirements.txt',
                    'hue/requirements.txt',
                    'parker/*',
                    'system_monitor/requirements.txt',
                    'system_monitor/requirements.txt',
                    'system_monitor/systemmonitor.py',
                    'system_monitor/systemmonitor.py',
                    'weather_current/*',
                    'weather_prediction/*']}

install_requires = \
['googleapis-common-protos>=1.5,<2.0',
 'grpcio-tools>=1.18,<2.0',
 'grpcio>=1.18,<2.0',
 'tlslite-ng>=0.7.5,<0.8.0']

setup_kwargs = {
    'name': 'pyxbos',
    'version': '0.2.4a1',
    'description': 'Python bindings for XBOS-flavored WAVEMQ and related services',
    'long_description': None,
    'author': 'Gabe Fierro',
    'author_email': 'gtfierro@cs.berkeley.edu',
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.5,<4.0',
}


setup(**setup_kwargs)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: nullabletypes.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='nullabletypes.proto',
  package='xbospb',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x13nullabletypes.proto\x12\x06xbospb\"\x16\n\x05Int32\x12\r\n\x05value\x18\x01 \x01(\x05\"\x16\n\x05Int64\x12\r\n\x05value\x18\x01 \x01(\x03\"\x17\n\x06Uint64\x12\r\n\x05value\x18\x01 \x01(\x04\"\x17\n\x06\x44ouble\x12\r\n\x05value\x18\x01 \x01(\x01\"\x15\n\x04\x42ool\x12\r\n\x05value\x18\x01 \x01(\x08\x62\x06proto3')
)




_INT32 = _descriptor.Descriptor(
  name='Int32',
  full_name='xbospb.Int32',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='xbospb.Int32.value', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=31,
  serialized_end=53,
)


_INT64 = _descriptor.Descriptor(
  name='Int64',
  full_name='xbospb.Int64',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='xbospb.Int64.value', index=0,
      number=1, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=55,
  serialized_end=77,
)


_UINT64 = _descriptor.Descriptor(
  name='Uint64',
  full_name='xbospb.Uint64',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='xbospb.Uint64.value', index=0,
      number=1, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=79,
  serialized_end=102,
)


_DOUBLE = _descriptor.Descriptor(
  name='Double',
  full_name='xbospb.Double',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='xbospb.Double.value', index=0,
      number=1, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=104,
  serialized_end=127,
)


_BOOL = _descriptor.Descriptor(
  name='Bool',
  full_name='xbospb.Bool',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='xbospb.Bool.value', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=129,
  serialized_end=150,
)

DESCRIPTOR.message_types_by_name['Int32'] = _INT32
DESCRIPTOR.message_types_by_name['Int64'] = _INT64
DESCRIPTOR.message_types_by_name['Uint64'] = _UINT64
DESCRIPTOR.message_types_by_name['Double'] = _DOUBLE
DESCRIPTOR.message_types_by_name['Bool'] = _BOOL
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Int32 = _reflection.GeneratedProtocolMessageType('Int32', (_message.Message,), dict(
  DESCRIPTOR = _INT32,
  __module__ = 'nullabletypes_pb2'
  # @@protoc_insertion_point(class_scope:xbospb.Int32)
  ))
_sym_db.RegisterMessage(Int32)

Int64 = _reflection.GeneratedProtocolMessageType('Int64', (_message.Message,), dict(
  DESCRIPTOR = _INT64,
  __module__ = 'nullabletypes_pb2'
  # @@protoc_insertion_point(class_scope:xbospb.Int64)
  ))
_sym_db.RegisterMessage(Int64)

Uint64 = _reflection.GeneratedProtocolMessageType('Uint64', (_message.Message,), dict(
  DESCRIPTOR = _UINT64,
  __module__ = 'nullabletypes_pb2'
  # @@protoc_insertion_point(class_scope:xbospb.Uint64)
  ))
_sym_db.RegisterMessage(Uint64)

Double = _reflection.GeneratedProtocolMessageType('Double', (_message.Message,), dict(
  DESCRIPTOR = _DOUBLE,
  __module__ = 'nullabletypes_pb2'
  # @@protoc_insertion_point(class_scope:xbospb.Double)
  ))
_sym_db.RegisterMessage(Double)

Bool = _reflection.GeneratedProtocolMessageType('Bool', (_message.Message,), dict(
  DESCRIPTOR = _BOOL,
  __module__ = 'nullabletypes_pb2'
  # @@protoc_insertion_point(class_scope:xbospb.Bool)
  ))
_sym_db.RegisterMessage(Bool)


# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

from . import grpcserver_pb2 as grpcserver__pb2


class TestStub(object):
  """dummy service
  """

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.TestUnary = channel.unary_unary(
        '/xbospb.Test/TestUnary',
        request_serializer=grpcserver__pb2.TestParams.SerializeToString,
        response_deserializer=grpcserver__pb2.TestResponse.FromString,
        )
    self.TestStream = channel.unary_stream(
        '/xbospb.Test/TestStream',
        request_serializer=grpcserver__pb2.TestParams.SerializeToString,
        response_deserializer=grpcserver__pb2.TestResponse.FromString,
        )


class TestServicer(object):
  """dummy service
  """

  def TestUnary(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')

  def TestStream(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_TestServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'TestUnary': grpc.unary_unary_rpc_method_handler(
          servicer.TestUnary,
          request_deserializer=grpcserver__pb2.TestParams.FromString,
          response_serializer=grpcserver__pb2.TestResponse.SerializeToString,
      ),
      'TestStream': grpc.unary_stream_rpc_method_handler(
          servicer.TestStream,
          request_deserializer=grpcserver__pb2.TestParams.FromString,
          response_serializer=grpcserver__pb2.TestResponse.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'xbospb.Test', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))



import os
import sys
import struct
import logging
import socket
import random
import time
import threading
import grpc
import pickle
import base64
from cryptography import x509
from cryptography.hazmat.backends import default_backend
from tlslite import TLSConnection
from pymortar import Result
from pymortar import mortar_pb2 as pymortar
from pymortar import mortar_pb2_grpc
from pymortar import RAW, MEAN, MIN, MAX, COUNT, SUM
from pyxbos.helloworld_pb2 import *
import pyxbos.wave.eapi_pb2 as eapi_pb2
import pyxbos.wave.eapi_pb2_grpc as eapi_pb2_grpc
#from pyxbos.wavemq_pb2 import *
#from pyxbos.wavemq_pb2_grpc import *
from pyxbos.grpcserver_pb2 import *
from pyxbos.exceptions import *
import asyncio

XBOS_PERMSET = base64.urlsafe_b64decode("GyC5wUUGKON6uC4gxuH6TpzU9vvuKHGeJa1jUr4G-j_NbA==")

class WAVEGRPCClient:
    def __init__(self, address_tuple, namespace, entityfile, grpcservice, proof_file='clientproof.pem', waved='localhost:410'):
        self.address_tuple = address_tuple
        self.ns = namespace
        self.grpcservice = grpcservice
        self.nsbytes = base64.urlsafe_b64decode(self.ns)
        self.entityfile = open(entityfile, 'rb').read()
        self.perspective = eapi_pb2.Perspective(
            entitySecret=eapi_pb2.EntitySecret(
                DER=self.entityfile
            )
        )
        self._listen_address = None
        self._ready = threading.Event()

        self.wave_channel = grpc.insecure_channel(waved)
        self.wave_client = eapi_pb2_grpc.WAVEStub(self.wave_channel)
        resp = self.wave_client.Inspect(eapi_pb2.InspectParams(
            content=self.entityfile,
        ))
        self.entityhash = resp.entity.hash


        self.proof_file = open('clientproof.pem','rb').read()
        resp = self.wave_client.VerifyProof(eapi_pb2.VerifyProofParams(
            proofDER=self.proof_file,
        ))
        self.sigresp = self.wave_client.Sign(eapi_pb2.SignParams(
            perspective=self.perspective,
            content=self.proof_file,
        ))

        # setup server
        self._server_thread = threading.Thread(target=self.get_client_connection, daemon=True)
        self._server_thread.start()

    def setup_connection(self):
        hdr = self.generate_peer_header()

        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        #sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        sock.connect(self.address_tuple)
        self.upstream_connection = TLSConnection(sock)
        hs = self.upstream_connection.handshakeClientCert()
        self.upstream_connection.write(self.nsbytes)
        self.upstream_connection.write(hdr)
        invalid = self.read_peer_header(self.upstream_connection)
        if invalid.message != '':
            raise Exception("GRPC Server sent invalid header or proof {0}".format(invalid))

    @property
    def listen_address(self):
        self._ready.wait()
        return "{0}:{1}".format(*self._listen_address)

    def get_client_connection(self):
        listen_port = 5005
        while True:
            listen_address = ('localhost', listen_port)
            server = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            try:
                server.bind(listen_address)
                self._listen_address = listen_address
                self._ready.set()
                break
            except Exception as e:
                print("Failed to listen on {0}".format(listen_address), e)
                time.sleep(1)
                listen_port += 1
        print("Listening on {0}".format(listen_address))
        server.listen(10)

        while True:
            client_socket, addr = server.accept()
            # reconnect to the GRPC server on each call
            self.setup_connection()

            # start a thread to talk to the remote host
            proxy_thread = threading.Thread(target=self.handle_client,
                                            args=(client_socket,), daemon=True)
            proxy_thread.start()

    def handle_client(self, client_socket):
        while True:
            try:
                local_buffer = receive_from(client_socket)
                if len(local_buffer):
                    self.upstream_connection.send(local_buffer)
                # receive back the response
                remote_buffer = receive_from(self.upstream_connection)
                if len(remote_buffer):
                    # send the response to the local socket
                    client_socket.send(remote_buffer)
                # if no more data on the either side, close the connections
                if not len(local_buffer) or not len(remote_buffer):
                    print("Done with call")
                    break
            finally:
                client_socket.close()
                self.upstream_connection.close()

    def generate_peer_header(self):
        buf = bytes()
        buf += self.entityhash
        buf += struct.pack('<H', len(self.sigresp.signature))
        buf += self.sigresp.signature
        buf += struct.pack('<I', len(self.proof_file))
        buf += self.proof_file
        return buf

    def read_peer_header(self, conn):
        entityhash = conn.read(max=34,min=34)
        sigsize = struct.unpack('<H', conn.read(max=2,min=2))[0]
        signature = conn.read(max=sigsize, min=sigsize)
        proofsize = struct.unpack('<I', conn.read(max=4, min=4))[0]
        proof = conn.read(max=proofsize, min=proofsize)
        #TODO verify this
        # TODO: need peer certificate
        cert = self.upstream_connection.session.serverCertChain.x509List[0].bytes

        c = x509.load_der_x509_certificate(cert, default_backend())

        vresp = self.wave_client.VerifySignature(eapi_pb2.VerifySignatureParams(
            signer=entityhash,
            signature=signature,
            content=c.signature,
        ))
        if vresp.error.message != "":
            return vresp.error

        proofresp = self.wave_client.VerifyProof(eapi_pb2.VerifyProofParams(
            proofDER=proof,
            subject=entityhash,
            requiredRTreePolicy=eapi_pb2.RTreePolicy(
                namespace=self.nsbytes,
                statements=[
                    eapi_pb2.RTreePolicyStatement(
                        permissionSet=XBOS_PERMSET,
                        permissions=["serve_grpc"],
                        resource=self.grpcservice,
                    ),
                ],
            )
        ))
        if proofresp.result == None:
            return "no proof"
        return proofresp.error

def receive_from(connection):
    buffer = b""
    try:
        connection.settimeout(1)
        while True:
            data = connection.recv(4096)
            if not data:
                break
            buffer += data
    except:
        pass
    return buffer

class MortarClient:
    def __init__(self, cfg):
        if 'wave' not in cfg:
            cfg['wave'] = 'localhost:410'
        if 'entity' not in cfg:
            if 'WAVE_DEFAULT_ENTITY' in os.environ:
                cfg['entity'] = os.environ['WAVE_DEFAULT_ENTITY']
            else:
                raise ConfigMissingError('entity', extra="And no WAVE_DEFAULT_ENTITY in environment")
        if 'prooffile' not in cfg:
            raise ConfigMissingError('prooffile')
        if 'namespace' not in cfg:
            raise ConfigMissingError('namespace')
        if 'address' not in cfg:
            raise ConfigMissingError('address')
        if 'grpcservice' not in cfg:
            raise ConfigMissingError('grpcservice')

        parts = cfg['address'].split(':')
        address_tuple = (parts[0], int(parts[1]))

        self._proxy = WAVEGRPCClient(address_tuple, cfg['namespace'], cfg['entity'], cfg['grpcservice'], proof_file=cfg['prooffile'], waved=cfg['wave'])
        channel = grpc.insecure_channel(self._proxy.listen_address)
        self._client = mortar_pb2_grpc.MortarStub(channel)

    def qualify(self, required_queries):
        arg = pymortar.QualifyRequest(required=required_queries)
        return self._client.Qualify(arg)
    
    def fetch(self, request):
        res = Result()
        for a in self._client.Fetch(request):
            if a.error != "":
                logging.error(a.error)
                break
            res._add(a)
        res._build()
        return res
